@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2015 Fredrik Mellbin
+//  Copyright (c) 2007-2016 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -23,25 +23,26 @@
 #include "../core/utils.h"
 
 #include <algorithm>
+#include <cassert>
 
-static AVPixelFormat CSNameToPIXFMT(const char *CSName, AVPixelFormat Default) {
+static AVPixelFormat CSNameToPIXFMT(const char *CSName, AVPixelFormat Default, bool HighBitDepth) {
 	if (!CSName)
 		return FFMS_PIX_FMT(NONE);
 	std::string s = CSName;
 	std::transform(s.begin(), s.end(), s.begin(), toupper);
 	if (s == "")
 		return Default;
-	if (s == "YUV9")
+	if (s == "YUV9" || s == "YUV410P8")
 		return FFMS_PIX_FMT(YUV410P);
-	if (s == "YV411")
+	if (s == "YV411" || s == "YUV411P8")
 		return FFMS_PIX_FMT(YUV411P);
-	if (s == "YV12")
+	if (s == "YV12" || s == "YUV420P8")
 		return FFMS_PIX_FMT(YUV420P);
-	if (s == "YV16")
+	if (s == "YV16" || s == "YUV422P8")
 		return FFMS_PIX_FMT(YUV422P);
-	if (s == "YV24")
+	if (s == "YV24" || s == "YUV444P8")
 		return FFMS_PIX_FMT(YUV444P);
-	if (s == "Y8")
+	if (s == "Y8" || s == "GRAY8")
 		return FFMS_PIX_FMT(GRAY8);
 	if (s == "YUY2")
 		return FFMS_PIX_FMT(YUYV422);
@@ -49,6 +50,44 @@ static AVPixelFormat CSNameToPIXFMT(const char *CSName, AVPixelFormat Default) {
 		return FFMS_PIX_FMT(BGR24);
 	if (s == "RGB32")
 		return FFMS_PIX_FMT(RGB32);
+    if (HighBitDepth) {
+        if (s == "YUVA420P8")
+            return FFMS_PIX_FMT(YUVA420P);
+        if (s == "YUVA422P8")
+            return FFMS_PIX_FMT(YUVA422P);
+        if (s == "YUVA444P8")
+            return FFMS_PIX_FMT(YUVA444P);
+        if (s == "YUV420P16")
+            return FFMS_PIX_FMT(YUV420P16);
+        if (s == "YUVA420P16")
+            return FFMS_PIX_FMT(YUVA420P16);
+        if (s == "YUV422P16")
+            return FFMS_PIX_FMT(YUV422P16);
+        if (s == "YUVA422P16")
+            return FFMS_PIX_FMT(YUVA422P16);
+        if (s == "YUV444P16")
+            return FFMS_PIX_FMT(YUV444P16);
+        if (s == "YUVA444P16")
+            return FFMS_PIX_FMT(YUVA444P16);
+        if (s == "YUV420P10")
+            return FFMS_PIX_FMT(YUV420P10);
+        if (s == "YUVA420P10")
+            return FFMS_PIX_FMT(YUVA420P10);
+        if (s == "YUV422P10")
+            return FFMS_PIX_FMT(YUV422P10);
+        if (s == "YUVA422P10")
+            return FFMS_PIX_FMT(YUVA422P10);
+        if (s == "YUV444P10")
+            return FFMS_PIX_FMT(YUV444P10);
+        if (s == "YUVA444P10")
+            return FFMS_PIX_FMT(YUVA444P10);
+        if (s == "RGBP16")
+            return FFMS_PIX_FMT(GBRP16);
+        if (s == "RGBAP16")
+            return FFMS_PIX_FMT(GBRAP16);
+        if (s == "Y16" || s == "GRAY16")
+            return FFMS_PIX_FMT(GRAY16);
+    }
 
 	return FFMS_PIX_FMT(NONE);
 }
@@ -64,6 +103,11 @@ AvisynthVideoSource::AvisynthVideoSource(const char *SourceFile, int Track, FFMS
 {
 	VI = {};
 
+    // check if the two functions we need for many bits are present
+    VI.pixel_type = VideoInfo::CS_Y16;
+    HighBitDepth = (VI.ComponentSize() == 2 && VI.IsY());
+    VI.pixel_type = VideoInfo::CS_UNKNOWN;
+
 	ErrorInfo E;
 	V = FFMS_CreateVideoSource(SourceFile, Track, Index, Threads, SeekMode, &E);
 	if (!V)
@@ -198,6 +242,20 @@ AvisynthVideoSource::~AvisynthVideoSource() {
 	FFMS_DestroyVideoSource(V);
 }
 
+static int GetSubSamplingH(const VideoInfo &vi) {
+    if (vi.IsYUV() && vi.IsPlanar())
+        return vi.GetPlaneHeightSubsampling(PLANAR_U);
+    else
+        return 0;
+}
+
+static int GetSubSamplingW(const VideoInfo &vi) {
+    if (vi.IsYUV() && vi.IsPlanar())
+        return vi.GetPlaneWidthSubsampling(PLANAR_U);
+    else
+        return 0;
+}
+
 void AvisynthVideoSource::InitOutputFormat(
 	int ResizeToWidth, int ResizeToHeight, const char *ResizerName,
 	const char *ConvertToFormatName, IScriptEnvironment *Env) {
@@ -208,25 +266,50 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (!F)
 		Env->ThrowError("FFVideoSource: %s", E.Buffer);
 
-	int TargetFormats[9];
-	TargetFormats[0] = FFMS_GetPixFmt("yuv410p");
-	TargetFormats[1] = FFMS_GetPixFmt("yuv411p");
-	TargetFormats[2] = FFMS_GetPixFmt("yuv420p");
-	TargetFormats[3] = FFMS_GetPixFmt("yuv422p");
-	TargetFormats[4] = FFMS_GetPixFmt("yuv444p");
-	TargetFormats[5] = FFMS_GetPixFmt("gray8");
-	TargetFormats[6] = FFMS_GetPixFmt("yuyv422");
-	TargetFormats[7] = FFMS_GetPixFmt("bgra");
-	TargetFormats[8] = -1;
+    std::vector<int> TargetFormats;
+    if (HighBitDepth) {
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv420p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva420p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv422p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva422p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv444p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva444p16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv420p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva420p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv422p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva422p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuv444p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva444p10"));
+        TargetFormats.push_back(FFMS_GetPixFmt("gbrp16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("gbrap16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("gray16"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva420p"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva422p"));
+        TargetFormats.push_back(FFMS_GetPixFmt("yuva444p"));
+    }
+    TargetFormats.push_back(FFMS_GetPixFmt("yuv410p"));
+    TargetFormats.push_back(FFMS_GetPixFmt("yuv411p"));
+    TargetFormats.push_back(FFMS_GetPixFmt("yuv420p"));
+    TargetFormats.push_back(FFMS_GetPixFmt("yuv422p"));
+    TargetFormats.push_back(FFMS_GetPixFmt("yuv444p"));
+    TargetFormats.push_back(FFMS_GetPixFmt("gray8"));
+    TargetFormats.push_back(FFMS_GetPixFmt("yuyv422"));
+    TargetFormats.push_back(FFMS_GetPixFmt("bgra"));
+
+    // Remove unsupported formats from list so they don't appear as an early termination
+    TargetFormats.erase(std::remove(TargetFormats.begin(), TargetFormats.end(), -1), TargetFormats.end());
+
+    TargetFormats.push_back(-1);
 
 	// PIX_FMT_NV21 is misused as a return value different to the defined ones in the function
-	AVPixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, FFMS_PIX_FMT(NV21));
+	AVPixelFormat TargetPixelFormat = CSNameToPIXFMT(ConvertToFormatName, FFMS_PIX_FMT(NV21), HighBitDepth);
 	if (TargetPixelFormat == FFMS_PIX_FMT(NONE))
 		Env->ThrowError("FFVideoSource: Invalid colorspace name specified");
 
 	if (TargetPixelFormat != FFMS_PIX_FMT(NV21)) {
-		TargetFormats[0] = TargetPixelFormat;
-		TargetFormats[1] = -1;
+        TargetFormats.clear();
+        TargetFormats.push_back(TargetPixelFormat);
+        TargetFormats.push_back(-1);
 	}
 
 	if (ResizeToWidth <= 0)
@@ -239,16 +322,17 @@ void AvisynthVideoSource::InitOutputFormat(
 	if (Resizer == 0)
 		Env->ThrowError("FFVideoSource: Invalid resizer name specified");
 
-	if (FFMS_SetOutputFormatV2(V, TargetFormats,
+	if (FFMS_SetOutputFormatV2(V, TargetFormats.data(),
 		ResizeToWidth, ResizeToHeight, Resizer, &E))
 		Env->ThrowError("FFVideoSource: No suitable output format found");
 
 	F = FFMS_GetFrame(V, 0, &E);
-	TargetFormats[0] = F->ConvertedPixelFormat;
-	TargetFormats[1] = -1;
+    TargetFormats.clear();
+    TargetFormats.push_back(F->ConvertedPixelFormat);
+    TargetFormats.push_back(-1);
 
 		// This trick is required to first get the "best" default format and then set only that format as the output
-	if (FFMS_SetOutputFormatV2(V, TargetFormats,
+	if (FFMS_SetOutputFormatV2(V, TargetFormats.data(),
 		ResizeToWidth, ResizeToHeight, Resizer, &E))
 		Env->ThrowError("FFVideoSource: No suitable output format found");
 
@@ -256,10 +340,16 @@ void AvisynthVideoSource::InitOutputFormat(
 
 	if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj420p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv420p"))
 		VI.pixel_type = VideoInfo::CS_I420;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva420p"))
+        VI.pixel_type = VideoInfo::CS_YUVA420;
     else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj422p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv422p"))
         VI.pixel_type = VideoInfo::CS_YV16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva422p"))
+        VI.pixel_type = VideoInfo::CS_YUVA422;
     else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuvj444p") || F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv444p"))
         VI.pixel_type = VideoInfo::CS_YV24;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva444p"))
+        VI.pixel_type = VideoInfo::CS_YUVA444;
     else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv411p"))
         VI.pixel_type = VideoInfo::CS_YV411;
     else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv410p"))
@@ -272,6 +362,36 @@ void AvisynthVideoSource::InitOutputFormat(
 		VI.pixel_type = VideoInfo::CS_BGR32;
 	else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("bgr24"))
 		VI.pixel_type = VideoInfo::CS_BGR24;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv420p16"))
+        VI.pixel_type = VideoInfo::CS_YUV420P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva420p16"))
+        VI.pixel_type = VideoInfo::CS_YUVA420P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv422p16"))
+        VI.pixel_type = VideoInfo::CS_YUV422P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva422p16"))
+        VI.pixel_type = VideoInfo::CS_YUVA422P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv444p16"))
+        VI.pixel_type = VideoInfo::CS_YUV444P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva444p16"))
+        VI.pixel_type = VideoInfo::CS_YUVA444P16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv420p10"))
+        VI.pixel_type = VideoInfo::CS_YUV420P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva420p10"))
+        VI.pixel_type = VideoInfo::CS_YUVA420P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv422p10"))
+        VI.pixel_type = VideoInfo::CS_YUV422P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva422p10"))
+        VI.pixel_type = VideoInfo::CS_YUVA422P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuv444p10"))
+        VI.pixel_type = VideoInfo::CS_YUV444P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("yuva444p10"))
+        VI.pixel_type = VideoInfo::CS_YUVA444P10;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("gbrp16"))
+        VI.pixel_type = VideoInfo::CS_RGBP16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("gbrap16"))
+        VI.pixel_type = VideoInfo::CS_RGBAP16;
+    else if (F->ConvertedPixelFormat == FFMS_GetPixFmt("gray16"))
+        VI.pixel_type = VideoInfo::CS_Y16;
 	else
 		Env->ThrowError("FFVideoSource: No suitable output format found");
 
@@ -293,45 +413,39 @@ void AvisynthVideoSource::InitOutputFormat(
 	VI.width = F->ScaledWidth;
 	VI.height = F->ScaledHeight;
 
-	// Crop to obey avisynth's even width/height requirements
-	if (VI.pixel_type == VideoInfo::CS_I420) {
-		VI.height -= VI.height & 1;
-		VI.width -= VI.width & 1;
-	}
-
-	if (VI.pixel_type == VideoInfo::CS_YUY2) {
-		VI.width -= VI.width & 1;
-	}
-
-	if (RFFMode > 0) {
-		VI.height -= VI.height & 1;
-	}
+	// Crop to obey subsampling width/height requirements
+    VI.width -= VI.width % (1 << GetSubSamplingW(VI));
+    VI.height -= VI.height % (1 << (GetSubSamplingH(VI) + (RFFMode > 0 ? 1 : 0)));
 }
 
-static void BlitPlane(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env, int Plane) {
-	int PlaneId = 1 << Plane;
+static void BlitPlane(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env, int Plane, int PlaneId) {
 	Env->BitBlt(Dst->GetWritePtr(PlaneId), Dst->GetPitch(PlaneId),
 		Frame->Data[Plane], Frame->Linesize[Plane],
 		Dst->GetRowSize(PlaneId), Dst->GetHeight(PlaneId));
 }
 
 void AvisynthVideoSource::OutputFrame(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env) {
-	if (VI.pixel_type == VideoInfo::CS_I420) {
-		BlitPlane(Frame, Dst, Env, 0);
-		BlitPlane(Frame, Dst, Env, 1);
-		BlitPlane(Frame, Dst, Env, 2);
+	if (VI.IsPlanar()) {
+		BlitPlane(Frame, Dst, Env, 0, VI.IsRGB() ? PLANAR_G : PLANAR_Y);
+        if (HighBitDepth ? !VI.IsY() : !VI.IsY8()) {
+            BlitPlane(Frame, Dst, Env, 1, VI.IsRGB() ? PLANAR_B : PLANAR_U);
+            BlitPlane(Frame, Dst, Env, 2, VI.IsRGB() ? PLANAR_R : PLANAR_V);
+        }
+        if (VI.IsYUVA() || VI.IsPlanarRGBA())
+            BlitPlane(Frame, Dst, Env, 3, PLANAR_A);
 	} else if (VI.IsYUY2()) {
-		BlitPlane(Frame, Dst, Env, 0);
-	} else { // RGB
+		BlitPlane(Frame, Dst, Env, 0, 0);
+    } else if (VI.IsRGB24() || VI.IsRGB32()) {
 		Env->BitBlt(
 			Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1), -Dst->GetPitch(),
 			Frame->Data[0], Frame->Linesize[0],
 			Dst->GetRowSize(), Dst->GetHeight());
-	}
+    } else {
+        assert(false);
+    }
 }
 
-static void BlitField(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env, int Plane, int Field) {
-	int PlaneId = 1 << Plane;
+static void BlitField(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironment *Env, int Plane, int PlaneId, int Field) {
 	Env->BitBlt(
 		Dst->GetWritePtr(PlaneId) + Dst->GetPitch(PlaneId) * Field, Dst->GetPitch(PlaneId) * 2,
 		Frame->Data[Plane] + Frame->Linesize[Plane] * Field, Frame->Linesize[Plane] * 2,
@@ -339,20 +453,25 @@ static void BlitField(const FFMS_Frame *Frame, PVideoFrame &Dst, IScriptEnvironm
 }
 
 void AvisynthVideoSource::OutputField(const FFMS_Frame *Frame, PVideoFrame &Dst, int Field, IScriptEnvironment *Env) {
-	const FFMS_Frame *SrcPicture = (Frame);
-
-	if (VI.pixel_type == VideoInfo::CS_I420) {
-		BlitField(Frame, Dst, Env, 0, Field);
-		BlitField(Frame, Dst, Env, 1, Field);
-		BlitField(Frame, Dst, Env, 2, Field);
+	const FFMS_Frame *SrcPicture = Frame;
+	if (VI.IsPlanar()) {
+		BlitField(Frame, Dst, Env, 0, VI.IsRGB() ? PLANAR_G : PLANAR_Y, Field);
+        if (HighBitDepth ? !VI.IsY() : !VI.IsY8()) {
+            BlitField(Frame, Dst, Env, 1, VI.IsRGB() ? PLANAR_B : PLANAR_U, Field);
+            BlitField(Frame, Dst, Env, 2, VI.IsRGB() ? PLANAR_R : PLANAR_V, Field);
+        }
+        if (VI.IsYUVA() || VI.IsPlanarRGBA())
+            BlitField(Frame, Dst, Env, 3, PLANAR_A, Field);
 	} else if (VI.IsYUY2()) {
-		BlitField(Frame, Dst, Env, 0, Field);
-	} else { // RGB
+		BlitField(Frame, Dst, Env, 0, 0, Field);
+	} else if (VI.IsRGB24() || VI.IsRGB32()) {
 		Env->BitBlt(
 			Dst->GetWritePtr() + Dst->GetPitch() * (Dst->GetHeight() - 1 - Field), -Dst->GetPitch() * 2,
 			SrcPicture->Data[0] + SrcPicture->Linesize[0] * Field, SrcPicture->Linesize[0] * 2,
 			Dst->GetRowSize(), Dst->GetHeight() / 2);
-	}
+    } else {
+        assert(false);
+    }
 }
 
 PVideoFrame AvisynthVideoSource::GetFrame(int n, IScriptEnvironment *Env) {
