@@ -130,12 +130,10 @@ public:
 typedef unknown_size<FFMS_ResampleContext, ffms_resample_alloc_context, ffms_resample_free> FFResampleContext;
 
 void ClearErrorInfo(FFMS_ErrorInfo *ErrorInfo);
-bool AudioFMTIsFloat(AVSampleFormat FMT);
 void InitNullPacket(AVPacket &pkt);
 void FillAP(FFMS_AudioProperties &AP, AVCodecContext *CTX, FFMS_Track &Frames);
 
-std::wstring widen_path(const char *s);
-void LAVFOpenFile(const char *SourceFile, AVFormatContext *&FormatContext);
+void LAVFOpenFile(const char *SourceFile, AVFormatContext *&FormatContext, int Track);
 
 void FlushBuffers(AVCodecContext *CodecContext);
 
