@@ -34,9 +34,7 @@ extern "C" {
 
 
 // swscale and pp-related functions
-int64_t GetSWSCPUFlags();
 SwsContext *GetSwsContext(int SrcW, int SrcH, AVPixelFormat SrcFormat, int SrcColorSpace, int SrcColorRange, int DstW, int DstH, AVPixelFormat DstFormat, int DstColorSpace, int DstColorRange, int64_t Flags);
-AVColorSpace GetAssumedColorSpace(int Width, int Height);
 
 // timebase-related functions
 void CorrectRationalFramerate(int *Num, int *Den);
